#!/usr/bin/perl -w
#
# Copyright (C) 2001,2002 asdfGroup.com Inc. All rights reserved.
#
# This library is a commercial product and it is subject to the license
# agreement that can be found at the following URL:
#   http://www.asdfGroup.com/license.html
# Please contact our staff if you will have any questions:
#   mailto:staff@asdfGroup.com
#

use strict;

sub HexDump2Bin {
    local $_ = join "", @_;
    #    hex  mc
    s/\x83\x20//g;			  
    #    hex
    s/(?<=[0-9a-fA-F]) (?=[0-9a-fA-F])//g;
    if (/^\s*\S+\s+(?:[0-9a-fA-F]{2})+(?:\s|$)/) {
	#      hex,    ...
	s/^\s*\S+\s+(\S+).*\n?/$1/mg;
    } elsif (/^\s*(?:[0-9a-fA-F]{2})+(?:\s|$)/) {
	# ...       hex,    ...
	s/^\s*(\S+).*\n?/$1/mg;
    } else {
	# ...   
	die "Unknown packet format:\n---cut---\n$_\n---cut---\n";
    }
    #  
    die "bad format (this should not happens) $_\n" unless /^(?:[0-9a-fA-F]{2})+$/;
    return pack "H*", $_;
}

sub Bin2HexDump {
    (local $_, my %o) = (shift, offs=>1,ascii=>1,group=>4,uc=>1,groups=>8,@_);
    #        
    my $groups = 0;
    $_ = join "", map {($o{uc} ? uc($_) : lc($_)) . ((($groups+=1)%=$o{groups})
	? " " : "\n")} unpack "(H$o{group})*", $_||"";
    #        
    s/^(.+)\Z/$1 . " " x ($o{groups}-1 + $o{group}*$o{groups} - length($1))/em;
    #  ascii  offs
    my ($offs, $count) = (0, $o{group}*$o{groups}/2);
    join "", 
	map {$o{offs} ? sprintf("0x%04X  %s",($offs+=$count)-$count, $_) : $_}
	map {s/[^\x20-\x7E\xC0-\xFE]/./g;"$_\n"}
	map {$o{ascii} ? $_."   ".(pack"H*",join"",split) : $_} split /\n/;
}

sub Bin2HexDiff {
    my ($bin1, $bin2, %o) = @_;
    # enable ansi colors only for terminals by default
    $o{ansi} = -t select() unless defined $o{ansi};
    my $diff = "";
    #    
    my @txt1 = split /\n/, Bin2HexDump($bin1, %o);
    my @txt2 = split /\n/, Bin2HexDump($bin2, %o);
    #   
    for (0 .. Min($#txt1,$#txt2)) {
	#   ,    
	$diff .= "  ".$txt1[$_]."\n", next if $txt1[$_] eq $txt2[$_];
	#  ,  
	if ($o{ansi}) {
	    #  
	    my @l1 = split //, $txt1[$_];
	    my @l2 = split //, $txt2[$_];
	    #    
	    for (grep {$l1[$_] ne $l2[$_]} 0 .. Min($#l1,$#l2)) {
		$l1[$_] = "\e[31m".$l1[$_]."\e[37m";
		$l2[$_] = "\e[32m".$l2[$_]."\e[37m";
	    }
	    $txt1[$_] = join "", @l1;
	    $txt2[$_] = join "", @l2;
	}
	#   
	$diff .= "- ".$txt1[$_]."\n";
	$diff .= "+ ".$txt2[$_]."\n";
    }
    #     (,    txt1  txt2)
    $diff .= ($txt1[$_] ? "- " : "+ ").$txt1[$_].$txt2[$_]."\n"
	for Min($#txt1,$#txt2)+1 .. Max($#txt1,$#txt2);
    return $diff;
}

1;
