#!/usr/bin/perl -w
#
# Copyright (C) 2001,2002 asdfGroup.com Inc. All rights reserved.
#
# This library is a commercial product and it is subject to the license
# agreement that can be found at the following URL:
#   http://www.asdfGroup.com/license.html
# Please contact our staff if you will have any questions:
#   mailto:staff@asdfGroup.com
#

use strict;

sub time () {
    eval "use Time::HiRes;";
    local $^W;
    if ($@) {
        my $SYS_gettimeofday = ( join("", map { Cat("/usr/include/$_.h") }
            qw(syscall sys/syscall bits/syscall asm/unistd) ) =~ 
            /^\s*#define\s*(?:__NR|SYS)_gettimeofday\s*(\d+)/m )[0];
        *POWER::lib::time = $SYS_gettimeofday ? sub () {
                syscall $SYS_gettimeofday, my $data=pack("ll"), 0;
                my ($sec, $msec) = unpack "ll", $data;
                return $sec+$msec/1000000;
            } : sub () {
                return CORE::time();
            };
    } else {
        *POWER::lib::time = \&Time::HiRes::time;
    }
    goto &POWER::lib::time;
}

sub sleep {
    select undef, undef, undef, $_[0];
}

sub ConkOut {
    POWER::lib::_CronInit() if !defined($POWER::lib::TIME);
    my $wakeup = $POWER::lib::TIME + $_[0];
    CORE::sleep 1 while $POWER::lib::TIME < $wakeup;
}

# Some strange ideas:
# 1. If CRON will work without external process by using alarm() it will
#    possibly conflict with alarm() in user code.
# 2. If CRON will use compiled C program (or Inline::C) it will take less
#    memory, but we should save this program somethere.
# 3. If there exists running (by root) service which ioctl() to /dev/rtc
#    then we can use such service to check @CRON up to 8192 times/sec.
{ my $SIGpid = undef;
sub _CronInit {
    return if defined $POWER::lib::TIME;
    my $PARENTpid = $$;
    $SIG{"NUM".&SIG_CRON} = "IGNORE";
    die "Can't fork scheduler process: $!\n" unless defined($SIGpid=fork());
    exec {$^X} "scheduler ($PARENTpid)", "-e", "select undef,undef,undef," . 
	&MIN_INT." while kill(".&SIG_CRON.",$PARENTpid)" unless $SIGpid;
    $POWER::lib::TIME = 0;
    $SIG{"NUM".&SIG_CRON} = \&_CronExecute;
    sigaction(&SIG_CRON, undef, my $act=bless({}, "POSIX::SigAction"));
    $act->{FLAGS} |= &POSIX::SA_RESTART;
    sigaction(&SIG_CRON, $act);
}
END {kill 9, $SIGpid if $SIGpid} }

	     # name, left, sec, sub
{ 
my $TIME_ = 0;
my @CRON = ( "TIME", 1, 1, sub { $TIME_++; $POWER::lib::TIME = $TIME_*&MIN_INT } );
sub _CronExecute {
    for my $i ( 0 .. int($#CRON/4)) {
	next if --$CRON[$i*4+1];
	#         
	#    -        , 
	#    (    $CRON[$i*4+1]  
	# );    ,    "Deep recursion"
	$CRON[$i*4+3]->();
	$CRON[$i*4+1] = $CRON[$i*4+2];
    }
}
{ my $auto = 0;
sub CronSet {
    my ($name, $sec, $code) = @_;
    POWER::lib::_CronInit() if !defined($POWER::lib::TIME);
    local $SIG{"NUM".&SIG_CRON} = "IGNORE";
    die "'$sec' < ".&MIN_INT    if $sec < &MIN_INT;
    die "'$code' isn't CODE ref"    if ref($code) ne "CODE";
    $name="__AUTO__".($auto++) unless defined $name;
    my $i = (grep {$CRON[$_] eq $name} map {$_*4} 0..int($#CRON/4))[0] || @CRON;
    splice @CRON, $i, 4, ($name, int($sec*100), int($sec*100), $code);
    return $name;
}}
sub CronDel { # name
    local $SIG{"NUM".&SIG_CRON} = "IGNORE";
    my $i = (grep {$CRON[$_] eq $_[0]} map {$_*4} 0..int($#CRON/4))[0] || @CRON;
    splice @CRON, $i, 4;
}}

1;
